@@ -52,7 +52,7 @@ QString qDBusInterfaceFromMetaObject(const QMetaObject *mo)
         if (interface.startsWith("QDBus"_L1)) {
             interface.prepend("org.qtproject.QtDBus."_L1);
         } else if (interface.startsWith(u'Q') &&
-                   interface.length() >= 2 && interface.at(1).isUpper()) {
+                   interface.size() >= 2 && interface.at(1).isUpper()) {
             // assume it's Qt
             interface.prepend("org.qtproject.Qt."_L1);
         } else if (!QCoreApplication::instance()||
@@ -103,7 +103,19 @@ bool qDBusInterfaceInObject(QObject *obj, const QString &interface_name)
 // sig must be the normalised signature for the method
 int qDBusParametersForMethod(const QMetaMethod &mm, QList<QMetaType> &metaTypes, QString &errorMsg)
 {
-    return qDBusParametersForMethod(mm.parameterTypes(), metaTypes, errorMsg);
+    QList<QByteArray> parameterTypes;
+    parameterTypes.reserve(mm.parameterCount());
+
+    // Not using QMetaMethod::parameterTypes() since we call QMetaType::fromName below
+    // where we need any typedefs resolved already.
+    for (int i = 0; i < mm.parameterCount(); ++i) {
+        QByteArray typeName = mm.parameterMetaType(i).name();
+        if (typeName.isEmpty())
+            typeName = mm.parameterTypeName(i);
+        parameterTypes.append(typeName);
+    }
+
+    return qDBusParametersForMethod(parameterTypes, metaTypes, errorMsg);
 }
 
 #endif // QT_BOOTSTRAPPED
@@ -117,10 +129,7 @@ int qDBusParametersForMethod(const QList<QByteArray> &parameterTypes, QList<QMet
     metaTypes.append(QMetaType());        // return type
     int inputCount = 0;
     bool seenMessage = false;
-    QList<QByteArray>::ConstIterator it = parameterTypes.constBegin();
-    QList<QByteArray>::ConstIterator end = parameterTypes.constEnd();
-    for ( ; it != end; ++it) {
-        QByteArray type = *it;
+    for (QByteArray type : parameterTypes) {
         if (type.endsWith('*')) {
             errorMsg = "Pointers are not supported: "_L1 + QLatin1StringView(type);
             return -1;
@@ -128,7 +137,7 @@ int qDBusParametersForMethod(const QList<QByteArray> &parameterTypes, QList<QMet
 
         if (type.endsWith('&')) {
             QByteArray basictype = type;
-            basictype.truncate(type.length() - 1);
+            basictype.truncate(type.size() - 1);
 
             QMetaType id = QMetaType::fromName(basictype);
             if (!id.isValid()) {
