@@ -1,42 +1,6 @@
-/****************************************************************************
-**
-** Copyright (C) 2020 The Qt Company Ltd.
-** Copyright (C) 2015 Olivier Goffart <ogoffart@woboq.com>
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the QtGui module of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2020 The Qt Company Ltd.
+// Copyright (C) 2015 Olivier Goffart <ogoffart@woboq.com>
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 #include "qicon.h"
 #include "qicon_p.h"
@@ -66,6 +30,8 @@
 #ifndef QT_NO_ICON
 QT_BEGIN_NAMESPACE
 
+using namespace Qt::StringLiterals;
+
 /*!
     \enum QIcon::Mode
 
@@ -101,7 +67,7 @@ QT_BEGIN_NAMESPACE
 
 static int nextSerialNumCounter()
 {
-    static QBasicAtomicInt serial = Q_BASIC_ATOMIC_INITIALIZER(0);
+    Q_CONSTINIT static QBasicAtomicInt serial = Q_BASIC_ATOMIC_INITIALIZER(0);
     return 1 + serial.fetchAndAddRelaxed(1);
 }
 
@@ -132,6 +98,11 @@ QIconPrivate::QIconPrivate(QIconEngine *e)
 {
 }
 
+void QIconPrivate::clearIconCache()
+{
+    qt_cleanup_icon_cache();
+}
+
 /*! \internal
     Computes the displayDevicePixelRatio for a pixmap.
 
@@ -226,7 +197,7 @@ static QPixmapIconEngineEntry *bestSizeScaleMatch(const QSize &size, qreal scale
 QPixmapIconEngineEntry *QPixmapIconEngine::tryMatch(const QSize &size, qreal scale, QIcon::Mode mode, QIcon::State state)
 {
     QPixmapIconEngineEntry *pe = nullptr;
-    for (int i = 0; i < pixmaps.count(); ++i)
+    for (int i = 0; i < pixmaps.size(); ++i)
         if (pixmaps.at(i).mode == mode && pixmaps.at(i).state == state) {
             if (pe)
                 pe = bestSizeScaleMatch(size, scale, &pixmaps[i], pe);
@@ -303,7 +274,7 @@ QPixmap QPixmapIconEngine::scaledPixmap(const QSize &size, QIcon::Mode mode, QIc
         pm = pe->pixmap;
 
     if (pm.isNull()) {
-        int idx = pixmaps.count();
+        int idx = pixmaps.size();
         while (--idx >= 0) {
             if (pe == &pixmaps.at(idx)) {
                 pixmaps.remove(idx);
@@ -320,7 +291,7 @@ QPixmap QPixmapIconEngine::scaledPixmap(const QSize &size, QIcon::Mode mode, QIc
     if (!actualSize.isNull() && (actualSize.width() > size.width() || actualSize.height() > size.height()))
         actualSize.scale(size, Qt::KeepAspectRatio);
 
-    QString key = QLatin1String("qt_")
+    QString key = "qt_"_L1
                   % HexString<quint64>(pm.cacheKey())
                   % HexString<uint>(pe ? pe->mode : QIcon::Normal)
                   % HexString<quint64>(QGuiApplication::palette().cacheKey())
@@ -450,7 +421,7 @@ void QPixmapIconEngine::addFile(const QString &fileName, const QSize &size, QIco
 {
     if (fileName.isEmpty())
         return;
-    const QString abs = fileName.startsWith(QLatin1Char(':')) ? fileName : QFileInfo(fileName).absoluteFilePath();
+    const QString abs = fileName.startsWith(u':') ? fileName : QFileInfo(fileName).absoluteFilePath();
     const bool ignoreSize = !size.isValid();
     ImageReader imageReader(abs);
     const QByteArray format = imageReader.format();
@@ -485,15 +456,20 @@ void QPixmapIconEngine::addFile(const QString &fileName, const QSize &size, QIco
             }
         }
     }
-    for (const QImage &i : qAsConst(icoImages))
+    for (const QImage &i : std::as_const(icoImages))
         pixmaps += QPixmapIconEngineEntry(abs, i, mode, state);
     if (icoImages.isEmpty() && !ignoreSize) // Add placeholder with the filename and empty pixmap for the size.
         pixmaps += QPixmapIconEngineEntry(abs, size, mode, state);
 }
 
+bool QPixmapIconEngine::isNull()
+{
+    return pixmaps.isEmpty();
+}
+
 QString QPixmapIconEngine::key() const
 {
-    return QLatin1String("QPixmapIconEngine");
+    return "QPixmapIconEngine"_L1;
 }
 
 QIconEngine *QPixmapIconEngine::clone() const
@@ -549,12 +525,12 @@ bool QPixmapIconEngine::write(QDataStream &out) const
     return true;
 }
 
-Q_GLOBAL_STATIC_WITH_ARGS(QFactoryLoader, loader,
-    (QIconEngineFactoryInterface_iid, QLatin1String("/iconengines"), Qt::CaseInsensitive))
+Q_GLOBAL_STATIC_WITH_ARGS(QFactoryLoader, iceLoader,
+    (QIconEngineFactoryInterface_iid, "/iconengines"_L1, Qt::CaseInsensitive))
 
 QFactoryLoader *qt_iconEngineFactoryLoader()
 {
-    return loader();
+    return iceLoader();
 }
 
 
@@ -628,9 +604,9 @@ QFactoryLoader *qt_iconEngineFactoryLoader()
   icons: via \l addFile() and \l fromTheme().
 
   \l addFile() is useful if you have your own custom directory structure and do
-  not need to use the \l {Icon Theme Specification}{freedesktop.org Icon Theme
-  Specification}. Icons created via this approach use Qt's \l {High Resolution
-  Versions of Images}{"@nx" high DPI syntax}.
+  not need to use the \l {Freedesktop Icon Theme Specification}. Icons
+  created via this approach use Qt's \l {High Resolution Versions of Images}
+  {"@nx" high DPI syntax}.
 
   Using \l fromTheme() is necessary if you plan on following the Icon Theme
   Specification. To make QIcon use the high DPI version of an image, add an
@@ -666,8 +642,6 @@ QFactoryLoader *qt_iconEngineFactoryLoader()
     │       └── appointment-new.png
     └── index.theme
   \endcode
-
-  \sa {fowler}{GUI Design Handbook: Iconic Label}, {Icons Example}
 */
 
 
@@ -1053,9 +1027,9 @@ void QIcon::addPixmap(const QPixmap &pixmap, Mode mode, State state)
 static QIconEngine *iconEngineFromSuffix(const QString &fileName, const QString &suffix)
 {
     if (!suffix.isEmpty()) {
-        const int index = loader()->indexOf(suffix);
+        const int index = iceLoader()->indexOf(suffix);
         if (index != -1) {
-            if (QIconEnginePlugin *factory = qobject_cast<QIconEnginePlugin*>(loader()->instance(index))) {
+            if (QIconEnginePlugin *factory = qobject_cast<QIconEnginePlugin*>(iceLoader()->instance(index))) {
                 return factory->create(fileName);
             }
         }
@@ -1137,10 +1111,9 @@ QList<QSize> QIcon::availableSizes(Mode mode, State state) const
     Returns the name used to create the icon, if available.
 
     Depending on the way the icon was created, it may have an associated
-    name. This is the case for icons created with fromTheme() or icons
-    using a QIconEngine which supports the QIconEngine::IconNameHook.
+    name. This is the case for icons created with fromTheme().
 
-    \sa fromTheme(), QIconEngine
+    \sa fromTheme(), QIconEngine::iconName()
 */
 QString QIcon::name() const
 {
@@ -1153,6 +1126,10 @@ QString QIcon::name() const
     \since 4.6
 
     Sets the search paths for icon themes to \a paths.
+
+    The content of \a paths should follow the theme format
+    documented by setThemeName().
+
     \sa themeSearchPaths(), fromTheme(), setThemeName()
 */
 void QIcon::setThemeSearchPaths(const QStringList &paths)
@@ -1161,20 +1138,14 @@ void QIcon::setThemeSearchPaths(const QStringList &paths)
 }
 
 /*!
-  \since 4.6
+    \since 4.6
 
-  Returns the search paths for icon themes.
+    Returns the search paths for icon themes.
 
-  The default value will depend on the platform:
+    The default search paths will be defined by the platform.
+    All platforms will also have the resource directory \c{:\icons} as a fallback.
 
-  On X11, the search path will use the XDG_DATA_DIRS environment
-  variable if available.
-
-  By default all platforms will have the resource directory
-  \c{:\icons} as a fallback. You can use "rcc -project" to generate a
-  resource file from your icon theme.
-
-  \sa setThemeSearchPaths(), fromTheme(), setThemeName()
+    \sa setThemeSearchPaths(), fromTheme(), setThemeName()
 */
 QStringList QIcon::themeSearchPaths()
 {
@@ -1186,7 +1157,13 @@ QStringList QIcon::themeSearchPaths()
 
     Returns the fallback search paths for icons.
 
-    The default value will depend on the platform.
+    The fallback search paths are consulted for standalone
+    icon files if the \l{themeName()}{current icon theme}
+    or \l{fallbackThemeName()}{fallback icon theme} do
+    not provide results for an icon lookup.
+
+    If not set, the fallback search paths will be defined
+    by the platform.
 
     \sa setFallbackSearchPaths(), themeSearchPaths()
 */
@@ -1200,7 +1177,12 @@ QStringList QIcon::fallbackSearchPaths()
 
     Sets the fallback search paths for icons to \a paths.
 
-    \note To add some path without replacing existing ones:
+    The fallback search paths are consulted for standalone
+    icon files if the \l{themeName()}{current icon theme}
+    or \l{fallbackThemeName()}{fallback icon theme} do
+    not provide results for an icon lookup.
+
+    For example:
 
     \snippet code/src_gui_image_qicon.cpp 5
 
@@ -1216,11 +1198,15 @@ void QIcon::setFallbackSearchPaths(const QStringList &paths)
 
     Sets the current icon theme to \a name.
 
-    The \a name should correspond to a directory name in the
-    themeSearchPath() containing an index.theme
-    file describing its contents.
+    The theme will be will be looked up in themeSearchPaths().
 
-    \sa themeSearchPaths(), themeName()
+    At the moment the only supported icon theme format is the
+    \l{Freedesktop Icon Theme Specification}. The \a name should
+    correspond to a directory name in the themeSearchPath()
+    containing an \c index.theme file describing its contents.
+
+    \sa themeSearchPaths(), themeName(),
+    {Freedesktop Icon Theme Specification}
 */
 void QIcon::setThemeName(const QString &name)
 {
@@ -1232,8 +1218,12 @@ void QIcon::setThemeName(const QString &name)
 
     Returns the name of the current icon theme.
 
-    On X11, the current icon theme depends on your desktop
-    settings. On other platforms it is not set by default.
+    If not set, the current icon theme will be defined by the
+    platform.
+
+    \note Platform icon themes are only implemented on
+    \l{Freedesktop} based systems at the moment, and the
+    icon theme depends on your desktop settings.
 
     \sa setThemeName(), themeSearchPaths(), fromTheme(),
     hasThemeIcon()
@@ -1248,8 +1238,12 @@ QString QIcon::themeName()
 
     Returns the name of the fallback icon theme.
 
-    On X11, if not set, the fallback icon theme depends on your desktop
-    settings. On other platforms it is not set by default.
+    If not set, the fallback icon theme will be defined by the
+    platform.
+
+    \note Platform fallback icon themes are only implemented on
+    \l{Freedesktop} based systems at the moment, and the
+    icon theme depends on your desktop settings.
 
     \sa setFallbackThemeName(), themeName()
 */
@@ -1263,12 +1257,16 @@ QString QIcon::fallbackThemeName()
 
     Sets the fallback icon theme to \a name.
 
-    The \a name should correspond to a directory name in the
-    themeSearchPath() containing an index.theme
-    file describing its contents.
+    The fallback icon theme is consulted for icons not provided by
+    the \l{themeName()}{current icon theme}, or if the \l{themeName()}
+    {current icon theme} does not exist.
 
-    \note This should be done before creating \l QGuiApplication, to ensure
-    correct initialization.
+    The \a name should correspond to theme in the same format
+    as documented by setThemeName(), and will be looked up
+    in themeSearchPaths().
+
+    \note Fallback icon themes should be set before creating
+    QGuiApplication, to ensure correct initialization.
 
     \sa fallbackThemeName(), themeSearchPaths(), themeName()
 */
@@ -1280,67 +1278,68 @@ void QIcon::setFallbackThemeName(const QString &name)
 /*!
     \since 4.6
 
-    Returns the QIcon corresponding to \a name in the current
-    icon theme.
+    Returns the QIcon corresponding to \a name in the
+    \l{themeName()}{current icon theme}.
 
-    The latest version of the freedesktop icon specification and naming
-    specification can be obtained here:
-
-    \list
-    \li \l{http://standards.freedesktop.org/icon-theme-spec/icon-theme-spec-latest.html}
-    \li \l{http://standards.freedesktop.org/icon-naming-spec/icon-naming-spec-latest.html}
-    \endlist
+    If the current theme does not provide an icon for \a name,
+    the \l{fallbackThemeName()}{fallback icon theme} is consulted,
+    before falling back to looking up standalone icon files in the
+    \l{QIcon::fallbackSearchPaths()}{fallback icon search path}.
 
     To fetch an icon from the current icon theme:
 
     \snippet code/src_gui_image_qicon.cpp 3
 
-    \note By default, only X11 will support themed icons. In order to
-    use themed icons on Mac and Windows, you will have to bundle a
-    compliant theme in one of your themeSearchPaths() and set the
-    appropriate themeName().
+    If an \l{themeName()}{icon theme} has not been explicitly
+    set via setThemeName() a platform defined icon theme will
+    be used.
 
-    \note Qt will make use of GTK's icon-theme.cache if present to speed up
-    the lookup. These caches can be generated using gtk-update-icon-cache:
-    \l{https://developer.gnome.org/gtk3/stable/gtk-update-icon-cache.html}.
+    \note Platform icon themes is only implemented on
+    \l{Freedesktop} based systems at the moment,
+    following the \l{Freedesktop Icon Naming Specification}.
+    In order to use themed icons on other platforms, you will have
+    to bundle a \l{setThemeName()}{compliant theme} in one of your
+    themeSearchPaths(), and set the appropriate themeName().
 
-    \note If an icon can't be found in the current theme, then it will be
-    searched in fallbackSearchPaths() as an unthemed icon.
-
-    \sa themeName(), setThemeName(), themeSearchPaths(), fallbackSearchPaths()
+    \sa themeName(), fallbackThemeName(), setThemeName(), themeSearchPaths(), fallbackSearchPaths(),
+        {Freedesktop Icon Naming Specification}
 */
 QIcon QIcon::fromTheme(const QString &name)
 {
-    QIcon icon;
 
-    if (qtIconCache()->contains(name)) {
-        icon = *qtIconCache()->object(name);
-    } else if (QDir::isAbsolutePath(name)) {
+    if (QIcon *cachedIcon = qtIconCache()->object(name))
+        return *cachedIcon;
+
+    if (QDir::isAbsolutePath(name))
         return QIcon(name);
-    } else {
-        QPlatformTheme * const platformTheme = QGuiApplicationPrivate::platformTheme();
-        bool hasUserTheme = QIconLoader::instance()->hasUserTheme();
-        QIconEngine * const engine = (platformTheme && !hasUserTheme) ? platformTheme->createIconEngine(name)
-                                                   : new QIconLoaderEngine(name);
-        icon = QIcon(engine);
-        if (!icon.isNull())
-            qtIconCache()->insert(name, new QIcon(icon));
-    }
 
+    QIcon icon(new QThemeIconEngine(name));
+    qtIconCache()->insert(name, new QIcon(icon));
     return icon;
 }
 
 /*!
     \overload
 
-    Returns the QIcon corresponding to \a name in the current
-    icon theme. If no such icon is found in the current theme
-    \a fallback is returned instead.
+    Returns the QIcon corresponding to \a name in the
+    \l{themeName()}{current icon theme}.
 
-    If you want to provide a guaranteed fallback for platforms that
-    do not support theme icons, you can use the second argument:
+    If the current theme does not provide an icon for \a name,
+    the \l{fallbackThemeName()}{fallback icon theme} is consulted,
+    before falling back to looking up standalone icon files in the
+    \l{QIcon::fallbackSearchPaths()}{fallback icon search path}.
+
+    If no icon is found \a fallback is returned.
+
+    This is useful to provide a guaranteed fallback, regardless of
+    whether the current set of icon themes and fallbacks paths
+    support the requested icon.
+
+    For example:
 
     \snippet code/src_gui_image_qicon.cpp 4
+
+    \sa fallbackThemeName(), fallbackSearchPaths()
 */
 QIcon QIcon::fromTheme(const QString &name, const QIcon &fallback)
 {
@@ -1356,7 +1355,8 @@ QIcon QIcon::fromTheme(const QString &name, const QIcon &fallback)
     \since 4.6
 
     Returns \c true if there is an icon available for \a name in the
-    current icon theme, otherwise returns \c false.
+    current icon theme or any of the fallbacks, as described by
+    fromTheme(), otherwise returns \c false.
 
     \sa themeSearchPaths(), fromTheme(), setThemeName()
 */
@@ -1458,16 +1458,16 @@ QDataStream &operator>>(QDataStream &s, QIcon &icon)
         icon = QIcon();
         QString key;
         s >> key;
-        if (key == QLatin1String("QPixmapIconEngine")) {
+        if (key == "QPixmapIconEngine"_L1) {
             icon.d = new QIconPrivate(new QPixmapIconEngine);
             icon.d->engine->read(s);
-        } else if (key == QLatin1String("QIconLoaderEngine")) {
-            icon.d = new QIconPrivate(new QIconLoaderEngine());
+        } else if (key == "QIconLoaderEngine"_L1 || key == "QThemeIconEngine"_L1) {
+            icon.d = new QIconPrivate(new QThemeIconEngine);
             icon.d->engine->read(s);
         } else {
-            const int index = loader()->indexOf(key);
+            const int index = iceLoader()->indexOf(key);
             if (index != -1) {
-                if (QIconEnginePlugin *factory = qobject_cast<QIconEnginePlugin*>(loader()->instance(index))) {
+                if (QIconEnginePlugin *factory = qobject_cast<QIconEnginePlugin*>(iceLoader()->instance(index))) {
                     if (QIconEngine *engine= factory->create()) {
                         icon.d = new QIconPrivate(engine);
                         engine->read(s);
@@ -1558,17 +1558,17 @@ QString qt_findAtNxFile(const QString &baseFileName, qreal targetDevicePixelRati
     if (disableNxImageLoading)
         return baseFileName;
 
-    int dotIndex = baseFileName.lastIndexOf(QLatin1Char('.'));
+    int dotIndex = baseFileName.lastIndexOf(u'.');
     if (dotIndex == -1) { /* no dot */
         dotIndex = baseFileName.size(); /* append */
-    } else if (dotIndex >= 2 && baseFileName[dotIndex - 1] == QLatin1Char('9')
-        && baseFileName[dotIndex - 2] == QLatin1Char('.')) {
+    } else if (dotIndex >= 2 && baseFileName[dotIndex - 1] == u'9'
+        && baseFileName[dotIndex - 2] == u'.') {
         // If the file has a .9.* (9-patch image) extension, we must ensure that the @nx goes before it.
         dotIndex -= 2;
     }
 
     QString atNxfileName = baseFileName;
-    atNxfileName.insert(dotIndex, QLatin1String("@2x"));
+    atNxfileName.insert(dotIndex, "@2x"_L1);
     // Check for @Nx, ..., @3x, @2x file versions,
     for (int n = qMin(qCeil(targetDevicePixelRatio), 9); n > 1; --n) {
         atNxfileName[dotIndex + 1] = QLatin1Char('0' + n);
