@@ -1,41 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2020 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the QtGui module of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2021 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 #include "qeglplatformcontext_p.h"
 #include "qeglconvenience_p.h"
@@ -44,7 +8,7 @@
 #include <QtGui/qopenglcontext.h>
 #include <QtCore/qdebug.h>
 
-#if defined(Q_OS_ANDROID) && !defined(Q_OS_ANDROID_EMBEDDED)
+#ifdef Q_OS_ANDROID
 #include <QtCore/private/qjnihelpers_p.h>
 #endif
 #ifndef Q_OS_WIN
@@ -155,6 +119,17 @@ QEGLPlatformContext::QEGLPlatformContext(const QSurfaceFormat &format, QPlatform
         }
     }
 
+#ifdef EGL_EXT_protected_content
+    if (format.testOption(QSurfaceFormat::ProtectedContent)) {
+        if (q_hasEglExtension(m_eglDisplay, "EGL_EXT_protected_content")) {
+            contextAttrs.append(EGL_PROTECTED_CONTENT_EXT);
+            contextAttrs.append(EGL_TRUE);
+        } else {
+            m_format.setOption(QSurfaceFormat::ProtectedContent, false);
+        }
+    }
+#endif
+
     // Special Options for OpenVG surfaces
     if (m_format.renderableType() == QSurfaceFormat::OpenVG) {
         contextAttrs.append(EGL_ALPHA_MASK_SIZE);
