@@ -37,12 +37,6 @@ using namespace QtMiscUtils;
 
 class QNetworkProxy;
 
-static inline bool isSeparator(char c)
-{
-    static const char separators[] = "()<>@,;:\\\"/[]?={}";
-    return isLWS(c) || strchr(separators, c) != nullptr;
-}
-
 // ### merge with nextField in cookiejar.cpp
 static QHash<QByteArray, QByteArray> parseHttpOptionHeader(const QByteArray &header)
 {
@@ -109,6 +103,11 @@ static QHash<QByteArray, QByteArray> parseHttpOptionHeader(const QByteArray &hea
                     ++pos;
                 }
             } else {
+                const auto isSeparator = [](char c) {
+                    static const char separators[] = "()<>@,;:\\\"/[]?={}";
+                    return isLWS(c) || strchr(separators, c) != nullptr;
+                };
+
                 // case: token
                 while (pos < header.size()) {
                     char c = header.at(pos);
@@ -154,6 +153,9 @@ QNetworkReplyHttpImpl::QNetworkReplyHttpImpl(QNetworkAccessManager* const manage
         d->sslConfiguration.reset(new QSslConfiguration(request.sslConfiguration()));
 #endif
 
+    QObjectPrivate::connect(this, &QNetworkReplyHttpImpl::redirectAllowed, d,
+            &QNetworkReplyHttpImplPrivate::followRedirect, Qt::QueuedConnection);
+
     // FIXME Later maybe set to Unbuffered, especially if it is zerocopy or from cache?
     QIODevice::open(QIODevice::ReadOnly);
 
@@ -501,6 +503,13 @@ bool QNetworkReplyHttpImplPrivate::loadFromCacheIfAllowed(QHttpNetworkRequest &h
     QNetworkHeadersPrivate::RawHeadersList::ConstIterator it;
     cacheHeaders.setAllRawHeaders(metaData.rawHeaders());
 
+    it = cacheHeaders.findRawHeader("content-length");
+    if (it != cacheHeaders.rawHeaders.constEnd()) {
+        std::unique_ptr<QIODevice> data(nc->data(httpRequest.url()));
+        if (!data || data->size() < it->second.toLongLong())
+            return false; // The data is smaller than the content-length specified
+    }
+
     it = cacheHeaders.findRawHeader("etag");
     if (it != cacheHeaders.rawHeaders.constEnd())
         httpRequest.setHeaderField("If-None-Match", it->second);
@@ -864,9 +873,6 @@ void QNetworkReplyHttpImplPrivate::postRequest(const QNetworkRequest &newHttpReq
                 q, SLOT(onRedirected(QUrl,int,int)),
                 Qt::QueuedConnection);
 
-        QObject::connect(q, SIGNAL(redirectAllowed()), q, SLOT(followRedirect()),
-                         Qt::QueuedConnection);
-
 #ifndef QT_NO_SSL
         QObject::connect(delegate, SIGNAL(sslConfigurationChanged(QSslConfiguration)),
                 q, SLOT(replySslConfigurationChanged(QSslConfiguration)),
