@@ -432,8 +432,8 @@ BenchmarkResult BenchmarkResult::parse(QString const& line, QString* error)
         // format:
         //  "function","[globaltag:]tag","metric",value_per_iteration,total,iterations
         QStringList split = line.split(',');
-        if (split.count() != 6) {
-            if (error) *error = QString("Wrong number of columns (%1)").arg(split.count());
+        if (split.size() != 6) {
+            if (error) *error = QString("Wrong number of columns (%1)").arg(split.size());
             return out;
         }
 
@@ -459,11 +459,13 @@ BenchmarkResult BenchmarkResult::parse(QString const& line, QString* error)
     // This code avoids using a QRegExp because QRegExp might be broken.
     // Sample format: 4,000 msec per iteration (total: 4,000, iterations: 1)
 
-    QString sFirstNumber;
-    while (!remaining.isEmpty() && !remaining.at(0).isSpace()) {
-        sFirstNumber += remaining.at(0);
-        remaining.remove(0,1);
-    }
+    const auto begin = remaining.cbegin();
+    auto it = std::find_if(begin, remaining.cend(), [](const auto ch) {
+        return ch.isSpace();
+    });
+    QString sFirstNumber{std::distance(begin, it), Qt::Uninitialized};
+    std::move(begin, it, sFirstNumber.begin());
+    remaining.erase(begin, it);
     remaining = remaining.trimmed();
 
     // 4,000 -> 4000
@@ -636,6 +638,11 @@ bool TestLogger::shouldIgnoreTest(const QString &test) const
         return true;
 #endif
 
+    if (!qEnvironmentVariableIsEmpty("WAYLAND_DISPLAY")) {
+        qDebug() << "TestLogger::shouldIgnoreTest() ignore" << test << "on wayland/xwayland!";
+        return true;
+    }
+
     // These tests are affected by timing and whether the CPU tick counter
     // is monotonically increasing. They won't work on some machines so
     // leave them off by default. Feel free to enable them for your own
@@ -758,27 +765,28 @@ void checkErrorOutput(const QString &test, const QByteArray &errorOutput)
         return;
 
 #ifdef Q_CC_MINGW
-    if (test == "blacklisted" // calls qFatal()
-        || test == "silent") // calls qFatal()
-#endif
+    if (test == "silent") // calls qFatal()
         return;
+#endif
 
 #ifdef Q_OS_WIN
     if (test == "crashes")
         return; // Complains about uncaught exception
 #endif
 
+#ifdef Q_OS_UNIX
+    if (test == "assert"
+        || test == "crashes"
+        || test == "failfetchtype"
+        || test == "faildatatype")
+    return; // Outputs "Received signal 6 (SIGABRT)"
+#endif
+
 #ifdef Q_OS_LINUX
-    // QEMU outputs to stderr about uncaught signals
-    if (QTestPrivate::isRunningArmOnX86() &&
-        (test == "assert"
-         || test == "blacklisted"
-         || test == "crashes"
-         || test == "faildatatype"
-         || test == "failfetchtype"
-         || test == "silent"
-        ))
-        return;
+    if (test == "silent") {
+        if (QTestPrivate::isRunningArmOnX86())
+            return;         // QEMU outputs to stderr about uncaught signals
+    }
 #endif
 
     INFO(errorOutput.toStdString());
@@ -921,6 +929,7 @@ static QProcessEnvironment testEnvironment()
         const bool preserveLibPath = qEnvironmentVariableIsSet("QT_PRESERVE_TESTLIB_PATH");
         foreach (const QString &key, systemEnvironment.keys()) {
             const bool useVariable = key == "PATH" || key == "QT_QPA_PLATFORM"
+                || key == "ASAN_OPTIONS"
 #if defined(Q_OS_QNX)
                 || key == "GRAPHICS_ROOT" || key == "TZ"
 #elif defined(Q_OS_UNIX)
@@ -965,8 +974,7 @@ TestProcessResult runTestProcess(const QString &test, const QStringList &argumen
     const bool expectedCrash = test == "assert" || test == "exceptionthrow"
         || test == "fetchbogus" || test == "crashedterminate"
         || test == "faildatatype" || test == "failfetchtype"
-        || test == "crashes" || test == "silent"
-        || test == "blacklisted" || test == "watchdog";
+        || test == "crashes" || test == "silent" || test == "watchdog";
 
     if (expectedCrash) {
         environment.insert("QTEST_DISABLE_CORE_DUMP", "1");
